@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2012 Canonical
+ * Copyright (C) 2011-2013 Canonical
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -44,6 +44,7 @@
 #include <linux/netlink.h>
 #include <linux/cn_proc.h>
 
+#define APP_NAME		"powerstat"
 #define MIN_RUN_DURATION	(5*60)		/* We recommend a run of 5 minutes */
 #define SAMPLE_DELAY		(10)		/* Delay between samples in seconds */
 #define ROLLING_AVERAGE		(120)		/* 2 minute rolling average for power usage calculation */
@@ -342,7 +343,7 @@ static void stats_clear(stats_t *stats)
 {
 	int i;
 
-	for (i=0; i<MAX_VALUES; i++) {
+	for (i = 0; i < MAX_VALUES; i++) {
 		stats->value[i] = 0.0;
 		stats->inaccurate[i] = false;
 	}
@@ -356,7 +357,7 @@ static void stats_clear_all(stats_t *stats, const int n)
 {
 	int i;
 
-	for (i=0; i<n; i++)
+	for (i = 0; i < n; i++)
 		stats_clear(&stats[i]);
 }
 
@@ -396,28 +397,38 @@ static int stats_read(stats_t *info)
 	return 0;
 }
 
+/*
+ *  On Nexus 4 we occasionally get idle time going backwards so
+ *  work around this by ensuring we don't get -ve deltas.
+ */
+#define SANE_STATS(s1, s2) ((s2) - (s1)) < 0.0 ? 0.0 : ((s2) - (s1))
+
 /*
  *  stats_gather()
  *	gather up delta between last stats and current to get
  * 	some form of per sample accounting calculated.
  */
-static void stats_gather(stats_t *s1, stats_t *s2, stats_t *res)
+static bool stats_gather(stats_t *s1, stats_t *s2, stats_t *res)
 {
 	double total;
 
-	res->value[CPU_USER]    = s2->value[CPU_USER]    - s1->value[CPU_USER];
-	res->value[CPU_NICE]    = s2->value[CPU_NICE]    - s1->value[CPU_NICE];
-	res->value[CPU_SYS]     = s2->value[CPU_SYS]     - s1->value[CPU_SYS];
-	res->value[CPU_IDLE]    = s2->value[CPU_IDLE]    - s1->value[CPU_IDLE];
-	res->value[CPU_IOWAIT]  = s2->value[CPU_IOWAIT]  - s1->value[CPU_IOWAIT];
-	res->value[CPU_IRQ]     = s2->value[CPU_IRQ]     - s1->value[CPU_IRQ];
-	res->value[CPU_SOFTIRQ] = s2->value[CPU_SOFTIRQ] - s1->value[CPU_SOFTIRQ];
-	res->value[CPU_CTXT]	= s2->value[CPU_CTXT]    - s1->value[CPU_CTXT];
-	res->value[CPU_INTR]	= s2->value[CPU_INTR]    - s1->value[CPU_INTR];
+	res->value[CPU_USER]    = SANE_STATS(s1->value[CPU_USER]   , s2->value[CPU_USER]);
+	res->value[CPU_NICE]    = SANE_STATS(s1->value[CPU_NICE]   , s2->value[CPU_NICE]);
+	res->value[CPU_SYS]     = SANE_STATS(s1->value[CPU_SYS]    , s2->value[CPU_SYS]);
+	res->value[CPU_IDLE]    = SANE_STATS(s1->value[CPU_IDLE]   , s2->value[CPU_IDLE]);
+	res->value[CPU_IOWAIT]  = SANE_STATS(s1->value[CPU_IOWAIT] , s2->value[CPU_IOWAIT]);
+	res->value[CPU_IRQ]     = SANE_STATS(s1->value[CPU_IRQ]    , s2->value[CPU_IRQ]);
+	res->value[CPU_SOFTIRQ] = SANE_STATS(s1->value[CPU_SOFTIRQ], s2->value[CPU_SOFTIRQ]);
+	res->value[CPU_CTXT]	= SANE_STATS(s1->value[CPU_CTXT]   , s2->value[CPU_CTXT]);
+	res->value[CPU_INTR]	= SANE_STATS(s1->value[CPU_INTR]   , s2->value[CPU_INTR]);
 
 	total = res->value[CPU_USER] + res->value[CPU_NICE] +
 		res->value[CPU_SYS] + res->value[CPU_IDLE] + res->value[CPU_IOWAIT];
 
+	/*  This should not happen, but we need to avoid division by zero or weird results */
+	if (total <= 0.0)
+		return false;
+
 	res->value[CPU_USER]   = (100.0 * res->value[CPU_USER]) / total;
 	res->value[CPU_NICE]   = (100.0 * res->value[CPU_NICE]) / total;
 	res->value[CPU_SYS]    = (100.0 * res->value[CPU_SYS]) / total;
@@ -429,6 +440,8 @@ static void stats_gather(stats_t *s1, stats_t *s2, stats_t *res)
 
 	res->value[CPU_PROCS_RUN] = s2->value[CPU_PROCS_RUN];
 	res->value[CPU_PROCS_BLK] = s2->value[CPU_PROCS_BLK];
+
+	return true;
 }
 
 /*
@@ -529,13 +542,13 @@ static void stats_average_stddev_min_max(stats_t *stats,
 	int j;
 	int valid;
 
-	for (j=0; j<MAX_VALUES; j++) {
+	for (j = 0; j < MAX_VALUES; j++) {
 		double total = 0.0;
 
 		max->value[j] = -1E6;
 		min->value[j] = 1E6;
 
-		for (valid=0,i=0; i<num; i++) {
+		for (valid = 0, i = 0; i < num; i++) {
 			if (!stats[i].inaccurate[j]) {
 				if (stats[i].value[j] > max->value[j])
 					max->value[j] = stats[i].value[j];
@@ -549,7 +562,7 @@ static void stats_average_stddev_min_max(stats_t *stats,
 		if (valid) {
 			average->value[j] = total / (double)valid;
 			total = 0.0;
-			for (i=0; i<num; i++) {
+			for (i = 0; i < num; i++) {
 				if (!stats[i].inaccurate[j]) {
 					double diff = (double)stats[i].value[j] - average->value[j];
 					diff = diff * diff;
@@ -676,8 +689,7 @@ static int power_rate_get_sys_fs(double *rate, bool *discharging, bool *inaccura
 	closedir(dir);
 
 	if (! *discharging) {
-		printf("Machine is indicating it is not discharging and hence "
-		       "we cannot measure power usage.\n");
+		printf("Machine is not discharging and hence we cannot measure power usage.\n");
 		return -1;
 	}
 
@@ -710,8 +722,8 @@ static int power_rate_get_sys_fs(double *rate, bool *discharging, bool *inaccura
 	 */
 	for (j = index, i = 0; i < MAX_MEASUREMENTS; i++) {
 		j--;
-		if (j<0)
-			j+= MAX_MEASUREMENTS;
+		if (j < 0)
+			j += MAX_MEASUREMENTS;
 
 		if (measurements[j].when) {
 			dw = measurements[j].value - total_capacity;
@@ -826,9 +838,13 @@ static int power_rate_get_proc_acpi(double *rate, bool *discharging, bool *inacc
 			sprintf(filename, "/proc/acpi/battery/%s/info", dirent->d_name);
 			if ((file = fopen(filename, "r")) != NULL) {
 				while (fgets(buffer, sizeof(buffer), file) != NULL) {
-					if (strstr(buffer, "design voltage:")) {
-						voltage = strtoull(ptr, NULL, 10) / 1000.0;
-						break;
+					ptr = strchr(buffer, ':');
+					if (ptr) {
+						ptr++;
+						if (strstr(buffer, "design voltage:")) {
+							voltage = strtoull(ptr, NULL, 10) / 1000.0;
+							break;
+						}
 					}
 				}
 				fclose(file);
@@ -875,8 +891,8 @@ static int power_rate_get_proc_acpi(double *rate, bool *discharging, bool *inacc
 	 */
 	for (j = index, i = 0; i < MAX_MEASUREMENTS; i++) {
 		j--;
-		if (j<0)
-			j+= MAX_MEASUREMENTS;
+		if (j < 0)
+			j += MAX_MEASUREMENTS;
 
 		if (measurements[j].when) {
 			dw = measurements[j].value - total_capacity;
@@ -953,7 +969,7 @@ static char *proc_info_get(const pid_t pid)
 	int i = proc_info_hash(pid);
 	int j;
 
-	for (j = 0; j<MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
+	for (j = 0; j < MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
 		if ((proc_info[i] != NULL) && (proc_info[i]->pid == pid))
 			return proc_info[i]->cmdline;
 	}
@@ -969,7 +985,7 @@ static void proc_info_free(const pid_t pid)
 	int i = proc_info_hash(pid);
 	int j;
 
-	for (j = 0; j<MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
+	for (j = 0; j < MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
 		if ((proc_info[i] != NULL) && (proc_info[i]->pid == pid)) {
 			free(proc_info[i]->cmdline);
 			free(proc_info[i]);
@@ -987,7 +1003,7 @@ static void proc_info_unload(void)
 {
 	int i;
 
-	for (i = 0; i<MAX_PIDS; i++) {
+	for (i = 0; i < MAX_PIDS; i++) {
 		if (proc_info[i] != NULL) {
 			free(proc_info[i]->cmdline);
 			free(proc_info[i]);
@@ -1026,7 +1042,7 @@ static int proc_info_add(const pid_t pid)
 	strcpy(info->cmdline, cmdline);
 
 	i = proc_info_hash(pid);
-	for (j = 0; j<MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
+	for (j = 0; j < MAX_PIDS; j++, i = (i + 1) % MAX_PIDS) {
 		if (proc_info[i] == NULL) {
 			proc_info[i] = info;
 			return 0;
@@ -1140,7 +1156,17 @@ static int monitor(const int sock)
 			time_now(tmbuffer, sizeof(tmbuffer));
 			gettimeofday(&t1, NULL);
 			stats_read(&s2);
-			stats_gather(&s1, &s2, &stats[readings]);
+
+			/*
+			 *  Total ticks was zero, something is broken, so re-sample
+			 */
+			if (!stats_gather(&s1, &s2, &stats[readings])) {
+				stats_clear(&stats[readings]);
+				stats_read(&s1);
+				gettimeofday(&t1, NULL);
+				redone |= OPTS_REDO_WHEN_NOT_IDLE;
+				continue;
+			}
 
 			if ((opts & OPTS_REDO_WHEN_NOT_IDLE) &&
 			    (stats[readings].value[CPU_IDLE] < (double)idle_threshold)) {
@@ -1282,6 +1308,7 @@ static int monitor(const int sock)
  */
 void show_help(char * const argv[])
 {
+	printf("%s, version %s\n\n", APP_NAME, VERSION);
 	printf("usage: %s [-b] [-d secs] [-h] [-i idle] [-p] [-r] [-s] [-z] [delay [count]]\n", argv[0]);
 	printf("\t-b redo a sample if a system is busy, considered less than %d%% CPU idle\n", IDLE_THRESHOLD);
 	printf("\t-d specify delay before starting, default is %d seconds\n", start_delay);
@@ -1387,7 +1414,7 @@ int main(int argc, char * const argv[])
 
 	if (start_delay > 0) {
 		/* Gather up initial data */
-		for (i=0; i < start_delay; i++) {
+		for (i = 0; i < start_delay; i++) {
 			printf("Waiting %d seconds before starting (gathering samples) \r", start_delay - i);
 			fflush(stdout);
 			if (power_rate_get(&dummy_rate, &discharging, &dummy_inaccurate) < 0)
